@@ -27,9 +27,10 @@ import fileinput
 import fnmatch
 import os
 import popen2
-import random
 import re
 import string
+import sys
+import tempfile
 import time
 
 
@@ -137,10 +138,10 @@ def unixdate(timesecs=None):
 
 
 def make_msgid(timesecs=None, pid=None):
-    """Return an rfc2822 compliant Message-ID string, composed of
-    date + process id + random integer + 'TMDA' + FQDN  e.g:
+    """Return an rfc2822 compliant Message-ID: string, composed of
+    seconds since the epoch in UTC + process id + 'TMDA' + FQDN. e.g:
     
-    <20020204183548.40803.32317.TMDA@nightshade.la.mastaler.com>
+    <1016659379.10104.TMDA@nightshade.la.mastaler.com>
 
     timesecs is optional, and if not given, the current time is used.
 
@@ -151,66 +152,54 @@ def make_msgid(timesecs=None, pid=None):
     if not pid:
         import Defaults
         pid = Defaults.PID
-    idhost = os.environ.get('QMAILIDHOST')
-    if not idhost:
-        idhost = gethostname()
-    utcdate = time.strftime('%Y%m%d%H%M%S', time.gmtime(timesecs))
-    randint = random.randrange(100000)
-    message_id = '<%s.%s.%s.TMDA@%s>' % (utcdate, pid, randint, idhost)
-    return message_id
+    idhost = os.environ.get('QMAILIDHOST') or gethostname()
+    msgid = '<%s.%s.TMDA@%s>' % (int(timesecs), pid, idhost)
+    return msgid
 
 
 def make_date(timesecs=None, localtime=1):
-    """Return an rfc2822 compliant Date string.  e.g,
-
-    Fri, 30 Nov 2001 04:06:11 -0700 (MST)
+    """Return an RFC 2822 compliant Date: string.
     
     timesecs is optional, and if not given, the current time is used.
 
     Optional localtime is a flag that when true, returns a date
-    relative to the local timezone instead of UTC.  This is the
-    default.
+    relative to the local timezone instead of UTC where possible.
+
+    JRM: Once the email mod is included within TMDA, we can nuke this
+    function and use email.Utils.formatdate in all cases.
     """
     if not timesecs:
         timesecs = time.time()
-    if localtime:
-        timetuple = time.localtime(timesecs)
-        tzname = time.tzname[timetuple[-1]]
-    else:
-        timetuple = time.gmtime(timesecs)
-        tzname = 'UTC'
     try:
-        import email.Utils
-        rfc2822date = email.Utils.formatdate(timesecs,localtime)
-        rfc2822date_tzname = '%s (%s)' % (rfc2822date, tzname)
+        from email.Utils import formatdate
+        datestr = formatdate(timesecs, localtime)
     except ImportError:
-        # This except block can be removed once Python >= 2.2 is required.
-        if localtime:
-            # Calculate timezone offset, based on whether the local zone has
-            # daylight savings time, and whether DST is in effect.
-            if time.daylight and timetuple[-1]:
-                offset = time.altzone
-            else:
-                offset = time.timezone
-            hours, minutes = divmod(abs(offset), 3600)
-            # Remember offset is in seconds west of UTC, but the timezone is in
-            # minutes east of UTC, so the signs differ.
-            if offset > 0:
-                sign = '-'
-            else:
-                sign = '+'
-            zone = '%s%02d%02d (%s)' % (sign, hours, minutes / 60, tzname)
-        else:
-            # Timezone offset is always -0000
-            zone = '%s (%s)' % ('-0000', tzname)
-        rfc2822date_tzname = '%s, %02d %s %04d %02d:%02d:%02d %s' % (
-            ['Mon', 'Tue', 'Wed', 'Thu', 'Fri', 'Sat', 'Sun'][timetuple[6]],
-            timetuple[2],
-            ['Jan', 'Feb', 'Mar', 'Apr', 'May', 'Jun',
-             'Jul', 'Aug', 'Sep', 'Oct', 'Nov', 'Dec'][timetuple[1] - 1],
-            timetuple[0], timetuple[3], timetuple[4], timetuple[5],
-        zone)
-    return rfc2822date_tzname
+        from rfc822 import formatdate
+        datestr = formatdate(timesecs)
+    return datestr
+
+
+def formataddr(pair):
+    """The inverse of parseaddr(), this takes a 2-tuple of the form
+    (realname, email_address) and returns the string value suitable
+    for an RFC 2822 From:, To: or Cc:.
+    
+    If the first element of pair is false, then the second element is
+    returned unmodified.
+
+    JRM: Once the email mod is included within TMDA, we can nuke this
+    function and use email.Utils.formataddr.
+    """
+    specialsre = re.compile(r'[][\()<>@,:;".]')
+    escapesre = re.compile(r'[][\()"]')
+    name, address = pair
+    if name:
+        quotes = ''
+        if specialsre.search(name):
+            quotes = '"'
+        name = escapesre.sub(r'\\\g<0>', name)
+        return '%s%s%s <%s>' % (quotes, name, quotes, address)
+    return address
 
 
 def file_to_dict(file, dict):
@@ -275,20 +264,24 @@ def pipecmd(command, *strings):
         cmdout.close()
         # Get exit status from the wait() member function.
         r = cmd.wait()
-        if err or r:
-            # If exit status is non-zero, raise an exception with data
-            # from childerr.
-            if r and os.WIFEXITED(r):
+        # If exit status is non-zero, raise an exception with data
+        # from childerr.
+        if r:
+            if os.WIFEXITED(r):
                 exitcode = 'exited %i' % os.WEXITSTATUS(r)
-                if os.WIFSIGNALED (r):
-                    exitsignal = 'signal %i' % os.WTERMSIG(r)
-                else:
-                    exitsignal = 'no signal'
+                exitsignal = ''
+            elif os.WIFSIGNALED(r):
+                exitcode = 'abnormal exit'
+                exitsignal = 'signal %i' % os.WTERMSIG(r)
             else:
+                # Stopped, etc.
                 exitcode = 'no exit?'
                 exitsignal = ''
             raise IOError, 'command "%s" %s %s (%s)' \
-                  % (command, exitcode, exitsignal, err)
+                  % (command, exitcode, exitsignal, err or '')
+        elif err:
+            # command wrote something to stderr.
+            print err
         if out:
             # command wrote something to stdout.
             print out
@@ -387,8 +380,6 @@ def build_cdb(filename):
 def build_dbm(filename):
     import anydbm
     import glob
-    import sys
-    import tempfile
 
     try:
         (dbmpath, dbmname) = os.path.split(filename)
@@ -421,14 +412,17 @@ def build_dbm(filename):
 def pickleit(object, file, bin=0):
     """Store object in a pickle file.
     Optional bin specifies whether to use binary or text pickle format."""
-    fp = open(file, 'w')
+    tempfile.tempdir = os.path.dirname(file)
+    tmpname = tempfile.mktemp()
+    fp = open(tmpname, 'w')
     cPickle.dump(object, fp, bin)
     fp.close()
+    os.rename(tmpname, file)
     return
 
 
 def unpickle(file):
-    """Retrieve and return object from the file file."""
+    """Retrieve and return object from file."""
     fp = open(file, 'r')
     object = cPickle.load(fp)
     fp.close()
@@ -548,23 +542,51 @@ def wraptext(text, column=70, honor_leading_ws=1):
 def maketext(templatefile, vardict):
     """Make some text from a template file.
 
-    Reads the `templatefile' which should be a full pathname, and does
-    string substitution by interpolating in the `localdict'.
+     Several locations are scanned for templatefile, in the following order:
+     1. The directory specified by tmda-filter's `-t' option.
+     2. Defaults.TEMPLATE_DIR
+     3. ../templates/
+     4. The package/RPM template directory.
+
+    The first match found stops the search.  In this way, you can
+    specialize templates at the desired level, or, if you use only the
+    default templates, you don't need to change anything.
+    
+    Once the templatefile is found, string substitution is performed
+    by interpolation in `localdict'.
 
     Based on code from Mailman
     <URL:http://www.gnu.org/software/mailman/mailman.html>
     Copyright (C) 1998,1999,2000,2001 by the Free Software Foundation, Inc.,
     and licensed under the GNU General Public License version 2.
     """
-    fp = open(templatefile)
-    template = fp.read()
-    fp.close()
     import Defaults
-    localdict = Defaults.__dict__.copy()
-    localdict.update(vardict)
-    text = template % localdict
-    return text
-
+    # Calculate the locations to scan.
+    searchdirs = []
+    searchdirs.append(os.environ.get('TMDA_TEMPLATE_DIR'))
+    searchdirs.append(Defaults.TEMPLATE_DIR)
+    searchdirs.append(Defaults.PARENTDIR + '/templates/')
+    searchdirs.append(sys.prefix + '/share/tmda/')
+    searchdirs.append('/etc/tmda/')
+    # Start scanning.
+    foundit = None
+    for dir in searchdirs:
+        if dir:
+            filename = os.path.join(dir, templatefile)
+            if os.path.exists(filename):
+                foundit = filename
+                break
+    if foundit is None:
+        raise IOError, "Can't find " + templatefile
+    else:
+        fp = open(foundit, 'r')
+        template = fp.read()
+        fp.close()
+        localdict = Defaults.__dict__.copy()
+        localdict.update(vardict)
+        text = template % localdict
+        return text
+        
 
 def filter_match(filename, recip, sender=None):
     """Check if the give e-mail addresses match lines in filename."""
