@@ -22,18 +22,30 @@
 """ Various versioning information."""
 
 
+import os
 import sys
 
 
 # TMDA version
-TMDA = "0.48"
+TMDA = "0.51"
 
 # Python version
+# e.g, 2.2 
 PYTHON = sys.version.split()[0]
 
-# Platform identifier
-PLATFORM = sys.platform
-
-# A summary of all the version identifiers.  e.g,
-# TMDA/0.46 (Python 2.2 on freebsd4)
+try:
+    # System information (sysname, nodename, release, version, machine)
+    # e.g, ('OSF1', 'spe147', 'V5.1', '1885', 'alpha')
+    UNAME = os.uname()
+    SYSNAME = UNAME[0].replace(' ', '_').replace('/', '_') # OSF1
+    MACHINE = UNAME[4].replace(' ', '_').replace('/', '_') # alpha
+    PLATFORM = SYSNAME + '/' + MACHINE  # OSF1/alpha
+except AttributeError:
+    # Fall back to using just sys.platform for PLATFORM if uname isn't
+    # available on this host.
+    UNAME = SYSNAME = MACHINE = None
+    PLATFORM = sys.platform             # osf1V5
+    
+# Summary of all the version identifiers
+# e.g, TMDA/0.49 (Python 2.2 on OSF1/alpha)
 ALL = "TMDA/%s (Python %s on %s)" % (TMDA, PYTHON, PLATFORM)
